      subroutine newh5(ier,id_f,fname)
!
! h5bits.f90 is an assembly of f90 routines intended to offer a simple f77
!            inteface to the hdf library for the principal file writing
!            functions        
!
! typically the character string arguments are last, simplifying certain
! link requirements
!
! The file has three levels, opening as root, which contains groups
! within which are (optionally) further groups, containing datasets 
! 
! As each component is opened the routine returns a "handle" identifier.
! The components are left open, since it is possible to add annotations
! (Attributes).  These are NAME,VALUE pairs with the value being real,
! integer or text; for text attributes there may be several lines of text.
!
! The dataset contents created here are either (multi-) dimensional arrays
! of integers or reals, or single lines of text.
!
! File
!      subroutine newh5(ier,id_f,fname)
!         Opens file fname; returns error ier and identifier id_f
!      subroutine endh5(ier,id_f)
!         Closes file id_f, returns error ier
!      subroutine oldh5(ier,id_f,fname,cmd
!         Opens file existing file fname returns id id_f if cmd is 'O'
!         Closes file id id_f if cmd is 'C'
! Groups
!      subroutine newgroup(ier,id_f, id_g, gname)
!         Opens group name gname in file id_f, returns error ier and 
!         group identifier id_g
!      subroutine endgroup(ier,id_g)
!         Closes group id_g
!
!
!
!Creating Datasets
!      subroutine newdsett(ier,id_f, id_d, text, dname)
!          Opens dataset name dname in existing placement id_f
!          plants one line of text 'text'
!          returns error ier and dataset id id_d  
!
!      subroutine newdsetf(ier,id_f, v, ir,idim, id_d,dname)
!          Opens dataset name dname in existing placement id_f
!          plants REAL array rank ir, dimensions in array idim(ir)
!          returns error ier and dataset id id_d  
!       
!
!      subroutine newdseti(ier,id_f, ii, ir,idim,id_d, dname)
!          Opens dataset name dname in existing placement id_f
!          plants INTEGER array rank ir, dimensions in array idim(ir)
!          returns error ier and dataset id id_d  
!
!      subroutine enddset(ier,id_d)
!          Closes dataset id_d
!
!      
!
! Attributes
!      subroutine newatt(ier,id_f,name,text)
!          Plants attribute name with text string  for entity id_f
!
!      subroutine newattn(ier,id_f,nl,name,text)
!          Plants attribute name with nl-line text string  for entity id_f
!
!      subroutine newatf(ier,id_f,name,fval)
!          Plants attribute name with  REAL value fval for entity id_f
!
!      subroutine newati(ier,id_f,name,ival)
!          Plants attribute name with  INTEGER value fval for entity id_f
!
!Reading Datasets
!      subroutine oldh5(ier,id_f,fname,cmd)
!       opens file fname cmd is "O" or "C" open/close
!
!      subroutine infods(ier,id_f,dsname,isiz,ir,idim,itype)
!        opens dsname returning size,rank,dims,type(0-C,1-I,2-F)
!
!      subroutine readds(ier,id_f,dsname,v,ir,idim,iitype)
!        returns contents of dsname in V
!        note idim(1) on input must be >= isiz
!      subroutine scanxs(fname,gname)
!        places names of datsets into 
!        common/scannam/dsnames*100(500)
!        common/scannml/lennam(500),nfound
!
!
! Ron Ghosh, Epsom, July-September 2012
!
! last modified 6 September

      use hdf5
      common/debh5/ideb
      character*(*) fname
      character*10 deb
      integer(HID_T) :: id_f
      integer ier
      ideb=0
      deb=' '
      call getenv('H5_DEBUG',deb)
      if(deb.ne.' ') ideb=1
! HID_T is 4
      call h5open_f(ier1)
! initialise Fortran interface
      call h5fcreate_f(fname, H5F_ACC_TRUNC_F, id_f, ier)
! create a new file using default parameters
      if(ideb.ne.0) write(*,*)'#newf ier2,ier,id_f:',fname,ier1,ier,id_f
      return
      end
      
      subroutine endh5(ier,id_f)
      use hdf5
      integer (HID_T) :: id_f
      call h5fclose_f(id_f,ier)
!  close FORTRAN interface
      call h5close_f(ier)
      return
      end
      
      subroutine newgroup(ier,id_f, id_g, gname)
! opens new group  gname can be relative or absolute ( name or /g1/name )
      use hdf5
      common/debh5/ideb
      character*(*) gname
      integer(HID_T) id_f, id_g
      call h5gcreate_f(id_f, gname, id_g, ier)
      if(ideb.ne.0) write(*,*) '#newg  id_f, id_g, ier:',gname,id_f,id_g,ier 
      return
      end
      
      subroutine endgroup(ier,id_g)
      use hdf5
      integer (HID_T) :: id_g
      call h5gclose_f(id_g,ier)
      return
      end
      
      subroutine newdsett(ier,id_f, id_d, text, dname)
      use hdf5
      common/debh5/ideb
      character*(*) dname
      character  :: text*(*)
      integer :: ir,id_d
      integer(SIZE_T) ltext
      integer(HSIZE_T), dimension(1) :: idimm
! MUST be dimension 1
      ir=1
      idimm(1)=1
!  create dataspace
      ltext=len(text)
!      if(ideb.ne.0) write(*,*) '#newt id_s,id_f, screate',dname,id_s,id_f,ier1
      call h5tcopy_f(H5T_NATIVE_CHARACTER, id_t, ier)
      call h5tset_size_f(id_t,ltext, ier)
      call h5screate_f(H5S_SCALAR_F, id_s, ier1 )
!  create dataset with default properties
      call h5dcreate_f(id_f, dname, id_t, id_s, id_d, ier2)
!      if(ideb.ne.0) write(*,*) '#newd id_s,id_f, space',dname,id_s,id_f,ier2
!  write data
      call h5dwrite_f(id_d, id_t, text, idimm , ier)
      if(ideb.ne.0) write(*,*) '#newt id_d,id_s,id_f, ier1,ier2,ier:',dname,id_d,id_s,id_f,ier1,ier2,ier
      return
      end

      subroutine newdsetf(ier,id_f, v, ir,idim, id_d,dname)
      use hdf5
      common/debh5/ideb
      character*(*) dname
      real  :: v(*)
      integer idim(*)
      integer :: ir,id_d
      integer(HSIZE_T), dimension(10) :: idimm
      irr=ir 
      do i=1,ir
        idimm(i)=idim(i)
      end do
!  create dataspace
      call h5screate_simple_f(ir , idimm, id_s, ier1 )
!      if(ideb.ne.0) write(*,*) '#newd id_s,id_f, screate',dname,id_s,id_f,ier1
!  create dataset with default properties
      call h5dcreate_f(id_f, dname, H5T_IEEE_F32LE, id_s, id_d, ier2)
!      if(ideb.ne.0) write(*,*) '#newd id_s,id_f, space',dname,id_s,id_f,ier2
!  write data
      call h5dwrite_f(id_d, H5T_IEEE_F32LE, v, idimm, ier)
      if(ideb.ne.0) write(*,*) '#newd id_d,id_s,id_f, ier1,ier2,ier:',dname,id_d,id_s,id_f,ier1,ier2,ier
      return
      end
      
      subroutine newdseti(ier,id_f, ii, ir,idim,id_d, dname)
      use hdf5
      common/debh5/ideb
      character*(*) dname
      integer ii(*)
      integer idim(*)
      integer :: ir
      integer(HSIZE_T), dimension(4) :: idimm 
      do i=1,4
       idimm(i)=0
      end do
      do i=1,ir
        idimm(i)=idim(i)
      end do
!  create dataspace
      call h5screate_simple_f(ir, idimm, id_s, ier1 )
!  create dataset with default properties
      call h5dcreate_f(id_f, dname, H5T_STD_I32LE, id_s, id_d, ier2)
!  write data
      call h5dwrite_f(id_d, H5T_STD_I32LE, ii, idimm, ier)
      if(ideb.ne.0) write(*,*) '#newd id_s,id_f, ier1,ier2,ier:',dname,id_d,id_s,id_f,ier1,ier2,ier
      return
      end

      subroutine enddset(ier,id_d)
      use hdf5
      common/debh5/ideb
      integer(HID_T) :: id_d
      call h5dclose_f(id_d, ier)
      if(ideb.ne.0) write(*,*)'#endd ier : ',ier
      return
      end
      
      subroutine newatt(ier,id_f,name,text)
      use hdf5
      common/debh5/ideb
      character*(*) name,text
      integer (HSIZE_T), DIMENSION(1) :: idim
      integer(SIZE_T) :: ltext
      integer(HSIZE_T), DIMENSION(1) :: adim = (/1/) !attribute dimension
      integer :: arank =1                            !attribute rank
      ltext=len(text)
 ! create space for attribute
      call h5screate_simple_f(arank,adim,id_s,ier)
 ! create data type for the attribute
      call h5tcopy_f(H5T_NATIVE_CHARACTER, id_t, ier)
      call h5tset_size_f(id_t,ltext, ier)
 ! create attribute
      call h5acreate_f(id_f, name, id_t, id_s, id_a, ier1)
      idim=1
      call h5awrite_f(id_a,id_t,text,adim,ier)
      call h5aclose_f(id_a,ier)
      if(ideb.ne.0) write(*,*) '#newat id_f,id_a,ier1,ier:',name,id_f,id_a,ier1,ier
      return
      end


      subroutine newattn(ier,id_f,nl,name,text)
  ! nl-line text attribute
      use hdf5
      common/debh5/ideb
      character*(*) name,text
      integer (HSIZE_T), DIMENSION(1) :: idim
      integer(SIZE_T) :: ltext
      integer :: arank = 1
      integer(HSIZE_T), DIMENSION(1) :: adim=(/1/)  !attribute dimension
      ltext=len(text)
      adim(1)=nl
 ! create space for attribute
      call h5screate_simple_f(arank,adim,id_s,ier)
 ! create data type for the attribute
      call h5tcopy_f(H5T_NATIVE_CHARACTER, id_t, ier)
      call h5tset_size_f(id_t,ltext, ier)
 ! create attribute
      call h5acreate_f(id_f, name, id_t, id_s, id_a, ier1)
      call h5awrite_f(id_a,id_t,text,adim,ier)
      call h5aclose_f(id_a,ier)
      if(ideb.ne.0) write(*,*) '#newat id_f,id_a,ier1,ier:',name,id_f,id_a,ier1,ier
      return
      end

      subroutine newatf(ier,id_f,name,fval)
      use hdf5
      common/debh5/ideb
      character*(*) name
      real fval
      integer (HSIZE_T), DIMENSION(1) :: idim
      integer(SIZE_T) :: lfval
      integer(HSIZE_T), DIMENSION(1) :: adim = (/1/) !attribute dimension
      integer :: arank =1                            !attribute rank
      lfval=4
 ! create space for attribute
      call h5screate_simple_f(arank,adim,id_s,ier)
 ! create data type for the attribute
      call h5tcopy_f(H5T_IEEE_F32LE, id_t, ier)
      call h5tset_size_f(id_t,lfval, ier)
 ! create attribute
      call h5acreate_f(id_f, name, id_t, id_s, id_a, ier1)
      idim=1
      call h5awrite_f(id_a,id_t,fval,adim,ier)
      call h5aclose_f(id_a,ier)
      if(ideb.ne.0) write(*,*) '#newaf id_f, ier1,ier:',name,id_f,id_a,ier1,ier
      return
      end

      subroutine newati(ier,id_f,name,ival)
      use hdf5
      common/debh5/ideb
      character*(*) name
      real fval
      integer (HSIZE_T), DIMENSION(1) :: idim
      integer(SIZE_T) :: lival
      integer(HSIZE_T), DIMENSION(1) :: adim = (/1/) !attribute dimension
      integer :: arank =1                            !attribute rank
      lival=4
 ! create space for attribute
      call h5screate_simple_f(arank,adim,id_s,ier)
 ! create data type for the attribute
      call h5tcopy_f(H5T_STD_I32LE, id_t, ier)
      call h5tset_size_f(id_t,lival, ier)
 ! create attribute
      call h5acreate_f(id_f, name, id_t, id_s, id_a, ier1)
      idim=1
      call h5awrite_f(id_a,id_t,ival,adim,ier)
      call h5aclose_f(id_a,ier)
      if(deb.ne.0) write(*,*) '#newai id_f, ier1,ier:',name,id_f,id_a,ier1,ier
      return
      end

      subroutine oldh5(ier,id_f,fname,cmd)
! open existing file if cmd is O
! close existing file if cmd is C
      use hdf5
      common/h5lims/idmax,namax,ioneh5,ioph5
      common/debh5/ideb
      character*(*) fname
      character*1 cmd
      character*10 deb
      integer(HID_T) :: id_f
      integer ier
      if(cmd.eq.'C') call h5fclose_f(id_f,ier)
      if(cmd.eq.'C') ioph5=0
      if(cmd.eq.'C') return
      if(ioph5.ne.0) call h5fclose_f(id_f,ier)
      ioph5=0
      ideb=0
      deb=' '
      call getenv('H5_DEBUG',deb)
      if(deb.ne.' ') ideb=1
! HID_T is 4
      call h5open_f(ier1)
! initialise Fortran interface doesn't matter if called more than once
      call h5fopen_f(fname, H5F_ACC_RDONLY_F, id_f, ier)
! open existing file RO file using default parameters
      if(ideb.ne.0) write(*,*)'#oldf ier2,ier,id_f:',fname,ier1,ier,id_f
      ioph5=1
      return
      end

      subroutine infods(ier,id_f,dsname,isiz,ir,idim,iitype)
! get nfo - number of objects
! returns rank and dimensions and type of dataset
      use hdf5
      common/debh5/ideb
      character*(*) dsname
      integer :: ir,isiz
      integer :: idim(10)
      integer(HID_T) :: id_t,id_d,id_f,id_s
      integer(SIZE_T) ::isize
      integer(HSIZE_T), dimension(10) :: idimm
      integer(HSIZE_T), dimension(10) :: maxdims
!      INTEGER(HID_T),  ALLOCATABLE DIMENSION(:) :: obj_ids
      integer(HID_T), dimension(20) :: obj_ids
      integer ier,itype
      ier=0
      do i=1,10
       idim(i)=0
      end do
!!      call h5fget_obj_count_f(id_f,H5F_OBJ_ALL_F,icount,ier)
!!      if(ideb.ne.0) write(*,*)'nfodis open object count', icount
!      allocate(obj_ids(icount),stat=ier)
!!      call h5fget_obj_ids_f(id_f,H5F_OBJ_FILE_F,20,obj_ids,ier)
!!! ier=H5LTfind_dataset_f(loc_id,dset_name)  !!!
! only returns objects actually open
! see h5ex_g_visitF03.f90
      call h5dopen_f(id_f,dsname,id_d,ier)
      if(ier.ne.0) return
! get datasets data type
      call h5dget_type_f(id_d,id_t,ier)
      if(ideb.ne.0) write(*,*) 'infods idtype ',id_t
      call h5tget_class_f(id_t,icl,ier)
      iitype=-1
      if(icl.eq.H5T_INTEGER_F) iitype=1
      if(icl.eq.H5T_FLOAT_F) iitype=2
      if(icl.eq.H5T_STRING_F) iitype=0
      if(ideb.ne.0) write(*,*) 'infodis type of data is ',iitype

      call h5tget_size_f(id_t,isize,ier)
      if(ideb.ne.0) write(*,*) 'infods storage size ',isize
      ier=-1
      isiz=isize
      
! dataset space dimensions
      call h5dget_space_f(id_d,id_s,ier)
      if(ideb.ne.0) write(*,*) 'infods idspace ',id_s

      call h5sget_simple_extent_ndims_f(id_s,irank,ier)
      if(ideb.ne.0) write(*,*) 'infods rank s ',irank
      call h5sget_simple_extent_dims_f(id_s,idimm,maxdims,ier)
! Get the datatype and its dimensions.
  !
      if(ideb.ne.0) write(*,*) 'infods dims ',(idimm(i),i=1,irank)
      do i=1,irank
        idim(i)=idimm(i)
      end do
      ir=irank
      call h5sclose_f(id_s,ier)
      call h5dclose_f(id_d,ier)
      return
      end
      
      subroutine readds(ier,id_f,dsname,v,ir,idim,iitype)
!     dsname is full name of dataset
!     returned values in v, dimension array idim
!     initially idim(1) has space available
      use hdf5
      common/debh5/ideb
      character*(*) dsname
      real  :: v(*)
      integer idim(10)
      integer :: ir, iitype
      integer(HID_T) :: id_t,id_d,id_f,id_s
      integer(HSIZE_T) ::isize
      integer(HSIZE_T), dimension(4) :: idimm
      integer(HSIZE_T), dimension(4) :: maxdims
! maxspace assume 4 byte storage
! idim(1) shows available storage in calling program
      ihave=idim(1)
      ier=0
      do i=1,4
       idim(i)=0
       idimm(i)=0
      end do
      call infods(ier,id_f,dsname,isiz,ir,idim,iitype)
      if(ier.ne.0) return
      if(ideb.ne.0) write(*,*) 'readds ir,isiz,idim1,iitype ',ir,isiz,idim,iitype
      maxspace=1
      ier=0
      do i=1,ir
       idimm(i)=idim(i)
       if(idim(i).gt.0) maxspace=maxspace*(idim(i))
      end do
      ineed=maxspace
      if(ideb.ne.0) write(*,*)'readds ihave ineed',ihave,ineed
      if(ineed.le.ihave) go to 100
      ier=-1
      return
100   call h5dopen_f(id_f,dsname,id_d,ier)
      call h5dget_type_f(id_d,id_t,ier)
      if(ideb.ne.0) write(*,*) 'readds idtype ',id_t
      ier=0
      if(ideb.ne.0) write(*,*)' readds calls h5dread idimm: ',idimm
      call h5dread_f(id_d,id_t,v,idimm,ier)
      ierr=ier
!     if buffer is scalar then idimm is not filled
!      call h5sclose_f(id_s,ier)
      call h5dclose_f(id_d,ier)
      if(iitype.eq.0) idim(1)=isiz
      ier=ierr
      return
      end

      subroutine scanxs(fname,gname)
      use HDF5
      common/debh5/ideb
      common/scannxc/igf
      common/scannam/dnames(500)
      common/scannml/nnam,nl(500)
      character fname*(*),gname*(*),namout*30
      character dnames*100
      integer(HID_T) :: id_f
      character*80 gname1,gname2
      do i=1,500
       nl(i)=0
      end do
      nnam=0
      lf=len_trim(fname)
      call h5open_f(ier)
      call h5fopen_f(fname(1:lf),H5F_ACC_RDONLY_F,id_f,ier)
       gname1=' '
      lg=len_trim(gname)
      call nextg(id_f,gname(1:lg),' ')
      call h5fclose_f(id_f,ier)
      return
      end

       recursive subroutine nextg(id_f,gname0,gname1)
       use HDF5
      common/scannam/dnames(500)
      common/scannml/nnam,nl(500)
      common/debh5/ideb
      common/scannxc/igf
      common/h5lims/idmax,namax,ioneh5,ioph5
      integer(HID_T) :: id_f
      character*100 gname,namout
      character dnames*100
      character*(*) gname0,gname1
      integer igf
      gname=' '
      gname=gname0//'/'//gname1
      lg=len_trim(gname)
      call h5gn_members_f(id_f,gname(1:lg),nmem,ier)
      if(ideb.ne.0.and.igf.eq.1) write(*,*) ' Group ',gname(2:lg),' has ',nmem,' members'
      do i=0,nmem-1
       namout=' '
       j=i+1
       call h5gget_obj_info_idx_f(id_f,gname(1:lg),i,namout,ityp,ier)
       lg1=len_trim(namout)
!       if(ideb.ne.0) then
!       if(ityp.eq.H5G_DATASET_F) write(*,*) gname(2:lg),'/',namout(1:lg1)
!       if(ityp.eq.H5G_GROUP_F.and.igf.eq.1) write(*,*)  namout(1:lg1),'     group'
!       end if
       if(ityp.eq.H5G_GROUP_F) call nextg(id_f,gname(1:lg),namout(1:lg1))
       if(ityp.eq.H5G_DATASET_F) then
       nnam=nnam+1
       dnames(nnam)=gname(1:lg)//'/'//namout(1:lg1)
       nll=lg+lg1+1
       nl(nnam)=nll
       if(ideb.ne.0) write(*,*) 'Dataset ',nnam,dnames(nnam)(2:nll)
       if(nnam.gt.namax) return
       end if
      end do
      return
      end
      block data h5bits
      common/debh5/ideb
      common/scannxc/igf
      common/h5lims/idmax,namax,ioneh5,ioph5
      data idmax/3276800/,namax/500/,ioneh5/0/,ioph5/0/
      data ideb/0/,igf/0/
      end
